@@ -61,7 +61,7 @@ extern "C" {
 /* PLVERSION_TAG: a string, normally "", but for example "rc1" */
 
 #ifndef PLVERSION
-#define PLVERSION 90324
+#define PLVERSION 90334
 #endif
 #ifndef PLVERSION_TAG
 #define PLVERSION_TAG ""
@@ -79,7 +79,7 @@ numbers are atomic type.
 #define PL_FLI_VERSION      2		/* PL_*() functions */
 #define	PL_REC_VERSION      3		/* PL_record_external(), fastrw */
 #define PL_QLF_LOADVERSION 68		/* load all versions later >= X */
-#define PL_QLF_VERSION     70		/* save version number */
+#define PL_QLF_VERSION     71		/* save version number */
 
 
 		 /*******************************
@@ -210,11 +210,7 @@ typedef _PLS(PL_local_data) *PL_engine_t; /* opaque engine handle */
 typedef _PLQ(word)	PL_atomic_t;	/* same a word */
 typedef uintptr_t	foreign_t;	/* return type of foreign functions */
 typedef wchar_t		pl_wchar_t;	/* Prolog wide character */
-#ifdef __cplusplus
-typedef void *		pl_function_t;      /* pass function as void* */
-#else
-typedef foreign_t	(*pl_function_t)(); /* foreign language functions */
-#endif
+typedef void *		pl_function_t;  /* pass function as void* */
 typedef uintptr_t	buf_mark_t;	/* buffer mark handle */
 
 #define fid_t PL_fid_t			/* avoid AIX name-clash */
@@ -1017,12 +1013,13 @@ PL_EXPORT(IOSTREAM *)	PL_open_resource(module_t m,
 					 const char *mode);
 
 PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
+PL_EXPORT(IOSTREAM *) _PL_stream(int which);
 #ifndef PL_KERNEL
-#define Suser_input     (_PL_streams()[0])
-#define Suser_output    (_PL_streams()[1])
-#define Suser_error     (_PL_streams()[2])
-#define Scurrent_input  (_PL_streams()[3])
-#define Scurrent_output (_PL_streams()[4])
+#define Suser_input     _PL_stream(0)
+#define Suser_output    _PL_stream(1)
+#define Suser_error     _PL_stream(2)
+#define Scurrent_input  _PL_stream(3)
+#define Scurrent_output _PL_stream(4)
 #endif
 
 #define PL_WRT_QUOTED		       0x01 /* quote atoms */
@@ -1117,20 +1114,14 @@ PL_EXPORT(int)		PL_dlclose(void *handle);
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 NOTE: the functions in this section are   not  documented, as as yet not
-adviced for public usage.  They  are   intended  to  provide an abstract
-interface for the GNU readline  interface   as  defined  in the readline
-package.
+adviced for public usage.  They  are   intended commandline editors such
+as GNU readline or libedit.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-					/* PL_dispatch() modes */
-#define PL_DISPATCH_NOWAIT    0		/* Dispatch only once */
-#define PL_DISPATCH_WAIT      1		/* Dispatch till input available */
-#define PL_DISPATCH_INSTALLED 2		/* dispatch function installed? */
 
-PL_EXPORT(int)		PL_dispatch(int fd, int wait);
 PL_EXPORT(void)		PL_add_to_protocol(const char *buf, size_t count);
-PL_EXPORT(char *)	PL_prompt_string(int fd);
-PL_EXPORT(void)		PL_write_prompt(int dowrite);
-PL_EXPORT(void)		PL_prompt_next(int fd);
+PL_EXPORT(char *)	PL_prompt_string(IOSTREAM *in);
+PL_EXPORT(void)		PL_write_prompt(bool dowrite);
+PL_EXPORT(void)		PL_prompt_next(IOSTREAM *in);
 PL_EXPORT(char *)	PL_atom_generator(const char *prefix, int state);
 PL_EXPORT(pl_wchar_t*)	PL_atom_generator_w(const pl_wchar_t *pref,
 					    pl_wchar_t *buffer,
@@ -1157,14 +1148,20 @@ PL_EXPORT(bool)		PL_linger(void *mem);
 		*             HOOKS		*
 		********************************/
 
+					/* PL_dispatch() modes */
+#define PL_DISPATCH_NOWAIT    0		/* Dispatch only once */
+#define PL_DISPATCH_WAIT      1		/* Dispatch till input available */
+#define PL_DISPATCH_INSTALLED 2		/* dispatch function installed? */
+
 #define PL_DISPATCH_INPUT   0		/* There is input available */
 #define PL_DISPATCH_TIMEOUT 1		/* Dispatch timeout */
 
-typedef int  (*PL_dispatch_hook_t)(int fd);
+typedef int  (*PL_dispatch_hook_t)(IOSTREAM *fd);
 typedef void (*PL_abort_hook_t)(void);
 typedef void (*PL_initialise_hook_t)(int argc, char **argv);
 typedef int  (*PL_agc_hook_t)(atom_t a);
 
+PL_EXPORT(bool)			PL_dispatch(IOSTREAM *fd, int wait);
 PL_EXPORT(PL_dispatch_hook_t)	PL_dispatch_hook(PL_dispatch_hook_t);
 PL_EXPORT(void)			PL_abort_hook(PL_abort_hook_t);
 PL_EXPORT(void)			PL_initialise_hook(PL_initialise_hook_t);
@@ -1341,18 +1338,19 @@ typedef struct
 
 PL_EXPORT(int)	PL_thread_self(void);	/* Prolog thread id (-1 if none) */
 PL_EXPORT(int)	PL_unify_thread_id(term_t t, int i);
-PL_EXPORT(int)	PL_get_thread_id_ex(term_t t, int *idp);
-PL_EXPORT(int)	PL_get_thread_alias(int tid, atom_t *alias);	/* Locks alias */
+PL_EXPORT(bool)	PL_get_thread_id_ex(term_t t, int *idp);
+PL_EXPORT(bool)	PL_get_thread_alias(int tid, atom_t *alias);	/* Locks alias */
 PL_EXPORT(int)	PL_thread_attach_engine(PL_thread_attr_t *attr);
 PL_EXPORT(bool)	PL_thread_destroy_engine(void);
-PL_EXPORT(int)	PL_thread_at_exit(void (*function)(void *),
+PL_EXPORT(bool)	PL_thread_at_exit(void (*function)(void *),
 				  void *closure,
-				  int global);
+				  bool global);
 PL_EXPORT(int)	PL_thread_raise(int tid, int sig);
 #if defined(_WINDOWS_) || defined(_WINDOWS_H)	/* <windows.h> is included */
-PL_EXPORT(int)	PL_w32thread_raise(DWORD dwTid, int sig);
-PL_EXPORT(int)	PL_wait_for_console_input(void *handle);
-PL_EXPORT(int)	PL_w32_wrap_ansi_console(void);
+PL_EXPORT(bool)	PL_w32thread_raise(DWORD dwTid, int sig);
+#endif
+#ifdef __WINDOWS__
+PL_EXPORT(bool)	PL_w32_wrap_ansi_console(void);
 PL_EXPORT(const char*) PL_w32_running_under_wine(void);
 #endif
 
@@ -1416,15 +1414,15 @@ PL_EXPORT(int)		PL_advance_hash_table_enum(hash_table_enum_t e,
 		 *******************************/
 
 typedef struct
-{ int	(*unify)(term_t t, void *handle);	/* implementation --> Prolog */
-  int   (*get)(term_t t, void **handle);	/* Prolog --> implementation */
-  void	(*activate)(int active);		/* (de)activate */
+{ bool	(*unify)(term_t t, void *handle);	/* implementation --> Prolog */
+  bool  (*get)(term_t t, void **handle);	/* Prolog --> implementation */
+  void	(*activate)(bool active);		/* (de)activate */
   void  (*release)(void *handle);		/* Release handle */
   void *dummy[4];				/* reserved */
   intptr_t	magic;				/* PROFTYPE_MAGIC */
 } PL_prof_type_t;
 
-PL_EXPORT(int)		PL_register_profile_type(PL_prof_type_t *type);
+PL_EXPORT(bool)		PL_register_profile_type(PL_prof_type_t *type);
 PL_EXPORT(void*)	PL_prof_call(void *handle, PL_prof_type_t *type);
 PL_EXPORT(void)		PL_prof_exit(void *node);
 
