@@ -54,11 +54,14 @@
             el_history_events/2,                % +Input, -Events
             el_add_history/2,                   % +Input, +Line
             el_write_history/2,                 % +Input, +FileName
-            el_read_history/2                   % +Input, +FileName
+            el_read_history/2,                  % +Input, +FileName
+
+	    el_version/1			% -Version:integer
           ]).
 :- autoload(library(apply),[maplist/2,maplist/3]).
 :- autoload(library(lists),[reverse/2,max_list/2,append/3,member/2]).
 :- autoload(library(solution_sequences),[call_nth/2]).
+:- autoload(library(option), [merge_options/3]).
 
 :- use_foreign_library(foreign(libedit4pl)).
 
@@ -81,7 +84,6 @@ streams and program the library.
 */
 
 el_wrap_if_ok :-
-    \+ current_prolog_flag(console_menu_version, qt),
     \+ current_prolog_flag(readline, readline),
     stream_property(user_input, tty(true)),
     !,
@@ -97,8 +99,17 @@ el_wrap_if_ok.
 %   low level predicates  that  allows   for  applying  and  programming
 %   libedit in non-standard situations.
 %
-%   The library is registered  with  _ProgName_   set  to  =swipl=  (see
+%   The library is registered  with  _ProgName_   set  to  `swipl`  (see
 %   el_wrap/4).
+%
+%   Options processed:
+%
+%     - pipes(+Boolean)
+%       Used by Epilog windows to indicate we are reading from a Windows
+%       named pipe in _overlapped_ mode.  Ignored on other platforms.
+%     - history(+Size)
+%       Size of the history.  Default is defined by the Prolog flag
+%       `history` or `100` if this flag is not defined.
 
 el_wrap :-
     el_wrap([]).
@@ -108,11 +119,22 @@ el_wrap(_) :-
     !.
 el_wrap(Options) :-
     stream_property(user_input, tty(true)), !,
-    el_wrap(swipl, user_input, user_output, user_error, Options),
+    findall(Opt, history_default(Opt), Defaults),
+    merge_options(Options, Defaults, Options1),
+    el_wrap(swipl, user_input, user_output, user_error, Options1),
     add_prolog_commands(user_input),
     forall(el_setup(user_input), true).
 el_wrap(_).
 
+history_default(history(Size)) :-
+    current_prolog_flag(history, Value),
+    (   integer(Value),
+        Value >= 0
+    ->  Size = Value
+    ;   Value == false
+    ->  Size = 0
+    ).
+
 add_prolog_commands(Input) :-
     el_addfn(Input, complete, 'Complete atoms and files', complete),
     el_addfn(Input, show_completions, 'List completions', show_completions),
@@ -179,7 +201,7 @@ el_wrap(ProgName, In, Out, Error) :-
 %   The predicate el_bind/2 is typically used   to bind commands defined
 %   using el_addfn/4. Note that the C proxy   function has only the last
 %   character of the command as context to find the Prolog binding. This
-%   implies we cannot both  bind  e.g.,  "^[?"  *and  "?"  to  a  Prolog
+%   implies we cannot both bind  e.g.,  "^[?"   _and_  "?"  to  a Prolog
 %   function.
 %
 %   @see editrc(5) for more information.
@@ -238,13 +260,37 @@ el_wrap(ProgName, In, Out, Error) :-
 %   Perform a generic action on the history. This provides an incomplete
 %   interface to history() from libedit.  Supported actions are:
 %
-%     * clear
-%     Clear the history.
-%     * setsize(+Integer)
-%     Set size of history to size elements.
-%     * setunique(+Boolean)
-%     Set flag that adjacent identical event strings should not be
-%     entered into the history.
+%     - clear
+%       Clear the history.
+%     - setsize(+Integer)
+%       Set size of history to size elements.
+%     - getsize(-Integer)
+%       Unify Integer with the maximum size of the history.  Note that
+%       this is _not_ the same as el_history() using ``H_GETSIZE``,
+%       which returns the number of currently saved events. The number
+%       of saved events may be computed from `first` and `last` or
+%       using el_history_events/2.
+%     - setunique(+Boolean)
+%       Set flag that adjacent identical event strings should not be
+%       entered into the history.
+%     - first(-Num, -String)
+%     - last(-Num, -String)
+%     - curr(-Num, -String)
+%     - prev(-Num, -String)
+%     - next(-Num, -String)
+%       Retrieve an event.  Num is the event number and String is the
+%       event string.  Note that `first` is the most recent event and
+%       `last` the oldest.
+%     - set(Num)
+%       Set the notion of _current_ to Num.
+%     - prev_str(+Search, -Num, -String)
+%     - next_str(+Search, -Num, -String)
+%       Retrieve the previous or next event whose String starts with
+%       Search.
+%     - event(+Num, -String)
+%       True when String represents event Num.   This is an extension to
+%       the history() API, retrieving a numbered event without changing
+%       the current notion.
 
 %!  el_history_events(+In:stream, -Events:list(pair)) is det.
 %
@@ -269,38 +315,74 @@ el_wrap(ProgName, In, Out, Error) :-
 %
 %   @arg File is a file specification for absolute_file_name/3.
 
+%!  el_version(-Version)
+%
+%   True when Version  is ``LIBEDIT_MAJOR*10000 + LIBEDIT_MINOR*100``.
+%   The  version is  generated from  the include  file ``histedit.h``,
+%   which implies that the actual version of the shared library may be
+%   different.
+
+%!  prolog:history(+Input, ?Action) is semidet.
+%
+%   Provide  the  plugable  interface  into   the  system  command  line
+%   management.
 
 :- multifile
     prolog:history/2.
 
+prolog:history(Input, enabled) :-
+    !,
+    el_wrapped(Input),
+    el_history(Input, getsize(Size)),
+    Size > 0.
 prolog:history(Input, add(Line)) :-
+    !,
     el_add_history(Input, Line).
 prolog:history(Input, load(File)) :-
-    el_read_history(Input, File).
+    !,
+    compat_read_history(Input, File).
 prolog:history(Input, save(File)) :-
+    !,
     el_write_history(Input, File).
-prolog:history(Input, load) :-
-    el_history_events(Input, Events),
-    load_history_events(Events).
+prolog:history(Input, events(Events)) :-
+    !,
+    el_history_events(Input, Events).
+prolog:history(Input, Command) :-
+    public_command(Command),
+    !,
+    el_history(Input, Command).
 
-%!  load_history_events(+Events)
+public_command(first(_Num, _String)).
+public_command(curr(_Num, _String)).
+public_command(event(_Num, _String)).
+public_command(prev_str(_Search, _Num, _String)).
+public_command(clear).
+
+%!  compat_read_history(+Input, +File) is det.
 %
-%   Load events into the history handling of `boot/history.pl`
+%   Read the saved history. This loads both  the LibEdit and old history
+%   format used by `swipl-win.exe` before migrating to SDL.
 
-load_history_events(Events) :-
-    '$reverse'(Events, RevEvents),
-    forall('$member'(Ev, RevEvents),
-           add_event(Ev)).
-
-add_event(Num-String) :-
-    remove_dot(String, String1),
-    '$save_history_event'(Num-String1).
-
-remove_dot(String0, String) :-
-    string_concat(String, ".", String0),
+compat_read_history(Input, File) :-
+    catch(el_read_history(Input, File), error(editline(_),_), fail),
     !.
-remove_dot(String, String).
+compat_read_history(Input, File) :-
+    access_file(File, read),
+    setup_call_cleanup(
+        open(File, read, In, [encoding(utf8)]),
+        read_old_history(Input, In),
+        close(In)),
+    !.
+compat_read_history(_, _).
 
+read_old_history(Input, From) :-
+    catch('$raw_read'(From, Line), error(_,_), fail),
+    (   Line == end_of_file
+    ->  true
+    ;   string_concat(Line, '.', Event),
+        el_add_history(Input, Event),
+        read_old_history(Input, From)
+    ).
 
 		 /*******************************
 		 *        ELECTRIC CARET	*
@@ -773,3 +855,13 @@ clipboard_content(Text) :-
                       autoload_call(
                           get(@(display), paste, primary, string(Text))))).
 clipboard_content("").
+
+
+                /*******************************
+                *           MESSAGE            *
+                *******************************/
+
+:- multifile prolog:error_message//1.
+
+prolog:error_message(editline(Msg)) -->
+    [ 'editline: ~s'-[Msg] ].
